// myAVR MK2 Driver for WinFACT (all versions)
// DELHPI 2009
// AVR Studio 4
//
// ===== Commands:
//
// ----- Write Output Pin
//
// Step 1: BORIS sends 4-Byte-Command to MK2
//
// Command structure: Wxyz
//
// x: 'B', 'C', 'D'  Port to write (char)
// y: 0 ... 7    Pin to write (byte)
// z: Pin Mode (char)
//    'O': Digital Output
//    'P': PWM Output
//
// Step 2: MK2 returns 'OK'
//
// Step 3: BORIS sends value to write (byte):
//          0 or 1 for Digital Output
//          0 ... 255 for PWM Output
//
// Step 4: MK2 returns 'OK'
//

// ----- Read Input Pin
//
// Step 1: BORIS sends 4-Byte-Command to MK2
//
// Command structure: Rxyz
//
// x: 'B', 'C', 'D'  Port to read (char)
// y: 0 ... 7    Pin to read (byte)
// z: Pin Mode
//    'I': Digital Input
//    'A': Analog Input
//
// Step 2:
//
// MK2 returns byte value 0 or 1 for Digital Input
// MK2 returns 2-byte value 0 ... 1023 for Analog Input (Low Byte first)
//


#include <avr/io.h>
 
#define BAUD 9600UL  // baudrate
 
// Calculations
#define UBRR_VAL ((F_CPU+BAUD*8)/(BAUD*16)-1)   // round
#define BAUD_REAL (F_CPU/(16*(UBRR_VAL+1)))     // actual baudrate
#define BAUD_ERROR ((BAUD_REAL*1000)/BAUD)      // baudrate error.
 
#if ((BAUD_ERROR<990) || (BAUD_ERROR>1010))
  #error baudrate error > 1%! 
#endif 

// Routines for UART

void uart_init(void)
{
  UBRRH = UBRR_VAL >> 8;
  UBRRL = UBRR_VAL & 0xFF;
 
  UCSRB |= (1<<TXEN);  // UART TX enabled
  UCSRB |= (1<<RXEN);  // UART RX enabled
  UCSRC = (1<<URSEL)|(1<<UCSZ1)|(1<<UCSZ0);  // Asynchron 8N1 
}

uint8_t uart_getc(void)
{
    while (!(UCSRA & (1<<RXC)))   // wait until character available
        ;
    return UDR;                   // return character
}

int uart_putc(unsigned char c)
{
    while (!(UCSRA & (1<<UDRE)))  // wait until transmission possible
    {
    }                             
 
    UDR = c;                      // send character
    return 0;
}


// Routines for ADC

void adc_init()
{
  ADMUX = (1 << REFS0);             // AVCC as reference
  ADCSRA = (1<<ADPS1) | (1<<ADPS2); // Pre-scaler 1/64
}

// Routines for PWM

void pwm_init()
{
	DDRB = (1 << PB1 )|(1 << PB2)|(1 << PB3);  // all PWM pins as outputs
	TCCR1A = (1<<WGM10);  // 8-bit-PWM for timer 1 A/B
	TCCR1B = (1 << CS10); // Pre-scaler 1 for timer 1
	TCCR2 |=  (1 << WGM20) | (1 << CS20);  // 8-bit-PWM for timer 2
	TCCR2 |= (1 << CS20);  // Pre-scaler 1 for timer 2
}


int main (void) {
 
  uart_init();
  adc_init();
  pwm_init(); 
     
  while(1)
  {	
    uint8_t c, x, y, z, lval;

    // ===== Read 4-Byte-Command

	c = uart_getc();
	x = uart_getc();
	y = uart_getc();
	z = uart_getc();

	// ===== Read I/O

	if (c == 'R')
      {
	    if (x == 'B')  // Port B
	      {
	        if (z == 'I')  // Digital Input
		      {
		        DDRB &= ~(1 << y);
				PORTB |= (1 << y); // Activate Pull-up
			    uart_putc((PINB & (1 << y)) != 0);
		      }
	      }
	    if (x == 'C')  // Port C
	      {
	        if (z == 'I')  // Digital Input
		      {
		        DDRC &= ~(1 << y);
				PORTC |= (1 << y); // Activate Pull-up
			    uart_putc((PINC & (1 << y)) != 0);
		      }
	        if (z == 'A')  // Analog Input
		      {
                ADCSRA |= (1<<ADEN);                     // activate ADC
                ADMUX = (ADMUX & ~(0x1F)) | (y & 0x1F);  // select channel
                ADCSRA |= (1<<ADSC);                     // start single conversion
                while (ADCSRA & (1<<ADSC) ) {            // wait until conversion is executed
                  }
                uart_putc(ADCL);                         // send low byte
				uart_putc(ADCH);                         // send high byte
                ADCSRA &= ~(1<<ADEN);                    // deactivate ADC
		      }
	      }
	    if (x == 'D')  // Port D
	      {
	        if (z == 'I')  // Digital Input
		      {
		        DDRD &= ~(1 << y);
				PORTD |= (1 << y); // Activate Pull-up
			    uart_putc((PIND & (1 << y)) != 0);
		      }
	      }
	  
	  }

	
	// ===== Write I/O

	if (c == 'W')
	  {
        uart_putc('O'); uart_putc('K');
	    lval = uart_getc();  // Read value to set
	    if (x == 'B')  // Port B
	      {
	        if (z == 'O')  // Digital Output
		      {
		        DDRB |= (1 << y);
				if (y == 1)
				  {
				    TCCR1A &= ~(1 << COM1A1);  // deactivate Timer 1 output A
				  }
				if (y == 2)
				  {
				    TCCR1A &= ~(1 << COM1B1);  // deactivate Timer 1 output B
				  }
				if (y == 3)
				  {
				    TCCR2 &= ~(1 << COM21);    // deactivate Timer 2 output
				  }
		        if (lval != 0)
		          {
		            PORTB |= (1 << y);
                  }
                else
		          {
		            PORTB &= ~(1 << y);
                  }
		      }
	        if (z == 'P')  // PWM Output
		      {
		        DDRB |= (1 << y);
				if (y == 1)
				  {
				    TCCR1A |= (1 << COM1A1);  // activate Timer 1 output A
					OCR1A = lval;
				  }
				if (y == 2)
				  {
				    TCCR1A |= (1 << COM1B1);  // activate Timer 1 output B
					OCR1B = lval;
				  }
				if (y == 3)
				  {
				    TCCR2 |= (1 << COM21);    // activate Timer 2 output
					OCR2 = lval;
				  }
		      }
      
	      }
	    if (x == 'C')  // Port C
	      {
	        if (z == 'O')  // Digital Output
		      {
		        DDRC |= (1 << y);
		        if (lval != 0)
		          {
		            PORTC |= (1 << y);
                  }
                else
		          {
		            PORTC &= ~(1 << y);
                  }
		      }
      
	      }
	    if (x == 'D')  // Port D
	      {
	        if (z == 'O')  // Digital Output
		      {
		        DDRD |= (1 << y);
		        if (lval != 0)
		          {
		            PORTD |= (1 << y);
                  }
                else
		          {
		            PORTD &= ~(1 << y);
                  }
		      }
      
	      }
        uart_putc('O'); uart_putc('K');
	  }
  }
}
